/*
 * File: rho_init.h
 * Date: 2011-11-17
 *
 * Evsiz (evsiz@this-voice.org)
 * Rho released under the Creative Commons NonCommercial ShareAlike License 3.0
 * See http://creativecommons.org for details
 *
 * Rho is a fork of the Linux port of MicroEMACS developed by Humberto
 * Ortiz-Zuazaga and Jean-Pierre Demailly. Their work forked the MicroEMACS
 * of Daniel M. Lawrence which was in turn a fork of uemacs by Dave G.
 * Conroy. MicroEmacs was copyrighted 1993 by Mr. Lawrence with a
 * stipulation that it could not be used commercially. Uemacs was released
 * by Mr. Conroy into the public domain in 1985. This file based on code by
 * Dave G. Conroy, Steve Wilhite and George Jones.
 *
 */


/* INITIALIZED GLOBAL DEFINITIONS */


char *execstr = NULL;          /* pointer to string to execute     */
char *fline = NULL;            /* dynamic return line              */
char *lastptr = NULL;          /* ptr to lastmesg[]                */
char *patmatch = NULL;         /* string that satisfies search cmd */

char bangline[NLINE] = "";     /* bang line for shell_buffer */
char compiler[NLINE] = "";     /* cmd for gcc            */
char fmtopts[NLINE] = "";      /* opts for fmt in shell  */
char sortopts[NLINE] = "";     /* opts for sort in shell */
char tagcmd[NLINE] = "";       /* cmd for ctags          */

char fmtlead[NPAT] = "";       /* format command lead-in chars   */
char golabel[NPAT] = "";       /* current line to go to          */
char lastmesg[NSTRING] = "";   /* last message posted            */
char lterm[NSTRING];           /* line terminators on file write */
char mainbuf[] = "scratch";    /* name of default buffer         */
char paralead[NPAT] = " \t";   /* paragraph leadin chars         */

const char errorm[] = "ERROR"; /* error literal    */
const char falsem[] = "FALSE"; /* false literal    */
const char truem[] = "TRUE";   /* true literal     */

const char modecode[] = "WC0EVOM0A0"; /* letters to represent modes   */

const char *modename[] = { "WRAP", "CMODE", "NULL", "EXACT", "VIEW",
  "OVER", "MAGIC", "NULL", "ASAVE", "NULL"
};

/************************ ORIGINAL MODES ****************************

const char modecode[] = "WCSEVOMYAR";
const char *modename[] = { "WRAP", "CMODE", "SPELL", "EXACT", "VIEW",
  "OVER", "MAGIC", "CRYPT", "ASAVE", "REP"};

*********************************************************************/

int *kbdend = &kbdm[0];        /* ptr to end of the keyboard */
int *kbdptr;                   /* current position in keyboard buf */
int abortc = CTRL | 'G';       /* current abort command char   */
int asave = 256;               /* global ASAVE size            */
int asavecount = 256;          /* count until next ASAVE       */
int charpending;               /* character pushed back        */
int clexec = FALSE;            /* command line execution flag  */
int cmdstatus = TRUE;          /* last command status          */
int cpending = FALSE;          /* input character pending?     */
int debug = FALSE;             /* macro debugging flag         */
int diagflag = FALSE;          /* diagonal cursor movement     */
int discmd = FALSE;            /* display command flag         */
int disinp = TRUE;             /* display input characters     */
int eexitflag = FALSE;         /* rho exit flag                */
int eexitval = 0;              /* and the exit return value    */
int eolexist = TRUE;           /* does clear to EOL exist?     */
int execlevel = 0;             /* execution IF level           */
int exec_error = FALSE;        /* macro execution error pending? */
int fillcol = 72;              /* Current fill column          */
int gflags = GFREAD;           /* global control flag          */
int gmode = 0;                 /* global editor mode           */
int hardtab = 8;               /* current hard tab size        */
int hjump = 1;                 /* horizontal jump size         */
int hscroll = TRUE;            /* horizontal scrolling flag    */
int hscrollbar = TRUE;         /* horizontal scroll bar flag   */
int isterm = CTRL | '[';       /* incremental-search terminating char */
int kbdmode = STOP;            /* current keyboard macro mode  */
int kbdm[NKBDM];               /* Macro                        */
int kbdrep = 0;                /* number of repetitions        */
int kill_index;                /* current index into kill ring */
int kskip[NRING];              /* # of bytes to skip in 1st kill chunk */
int kused[NRING];              /* # of bytes used in last kill chunk */
int lastkey = 0;               /* last keystoke                */
int lbound = 0;                /* leftmost column of current line being displayed */
int modeflag = TRUE;           /* display modelines flag       */
int mpresf = FALSE;            /* TRUE if message in last line */
int mstore = FALSE;            /* storing text to macro flag   */
int overlap = 2;               /* overlap when paging screens  */
int posflag = TRUE;            /* display point position       */
int predef = TRUE;             /*   "       "  default flag    */
int prefix = 0;                /* currently pending prefix bits */
int prenum = 0;                /*     "       "     numeric arg */
int quotec = CTRL | 'Q';       /* quote char during mlreply() */
int reptc =  CTRL | 'U';       /* current universal repeat char */
int revexist = FALSE;          /* does reverse video exist?    */
int rval = 0;                  /* return value of a subprocess */
int saveflag = 0;              /* Flags, saved with the $target var */
int searchtype = SRNORM;       /* current search style         */
int sgarbf = TRUE;             /* TRUE if screen is garbage    */
int softtab = 0;               /* current soft tab size (0: use hard tabs)  */
int sourcing= FALSE;           /* TRUE while exec.source is executing */
int sscroll = FALSE;           /* smooth scrolling enabled flag */
int sterm =  CTRL | '[';       /* search terminating character */
int taboff = 0;                /* tab offset for display       */
int ttcol = (int) HUGE;        /* Column location of HW cursor */
int ttrow = (int) HUGE;        /* Row location of HW cursor    */
int vscrollbar = TRUE;         /* vertical scroll bar flag */
int vtcol = 0;                 /* Column location of SW cursor */
int vtrow = 0;                 /* Row location of SW cursor    */
int wlflag = FALSE;            /* word list enabled flag       */
int xpos = 0;                  /* current column cursor is positioned to */
int yankflag = FALSE;          /* current yank style           */
int ypos = 0;                  /* current screen row        "          */
KILL *kbufh[NRING];            /* kill buffer header pointer   */
KILL *kbufp[NRING];            /* current kill buffer chunk pointer */
size_t flen = 0;               /* current length of fline      */
struct BUFFER *bstore = NULL;  /* buffer to store macro text to */
unsigned char wordlist[256];   /* characters considered "in words" */
WNDW *swindow = NULL;        /* saved window pointer         */

/*
 * The variables magical and rmagical determine if there
 * were actual metacharacters in the search and replace strings -
 * if not, then we don't have to use the slower MAGIC mode
 * search functions.
 */

int magical = FALSE;
int rmagical = FALSE;


/* directive name table: all directives */

const char *dname[] = {
  "if", "else", "endif", "goto", "return", "endm",
  "while", "endwhile", "break", "force"
};

/* directive lengths    */
const int dname_len[NUMDIRS] = { 2, 4, 5, 4, 6, 4, 5, 7, 5, 5 };

/*
 *  The envars are paired by location to the EV* in rho.h 
 *  where they are numbered defines.  
 */
char *envars[] = {
  "asave",        /* chars between auto-saves */
  "bangline",     /* bang lie for shell_buffer */
  "bufhook",      /* enter buffer switch hook */
  "cfname",       /* current file name */
  "cmdhook",      /* command loop hook */
  "compiler",     /* command for gcc compiler */
  "debug",        /* macro debugging */
  "exbhook",      /* exit buffer switch hook */
  "fillcol",      /* fill column for word wrap */
  "fmtopts",      /* opts for fmt in shell */
  "hardtab",      /* current hard tab size */
  "readhook",     /* read file execution hook */
  "replace",      /* replacement pattern */
  "rval",         /* child process return value */
  "search",       /* search pattern */
  "softtab",      /* current soft tab size */
  "sortopts",     /* opts for sort in shell */
  "status",       /* returns the status of the last command */
  "tagcmd",       /* command string for ctags */
  "time",         /* date and time */
  "wchars",       /* set of characters legal in words */
  "wraphook",     /* wrap word execution hook */
  "writehook",    /* write file hook */
};

#define NENVARS sizeof(envars) / sizeof(char *)
int nenvars = NENVARS;

const char *search_path[] = {
  BINDIR,
  SHRDIR,
  ""
};

#define	NPNAMES (int)(sizeof(search_path)/sizeof(char *))
int npnames = NPNAMES;
